#!/usr/bin/env python
# -*- coding: utf-8 -*-

import maya.cmds as cmds
import hmScripts.UI.hmUiCommonUtils as uiUtil

reload(uiUtil)

'''
DEVI TROVARE IL MODO DI PRENDERE LE OPZIONI ANCHE DALLO SPLIT E DAL BLEND... probabilmente bastera leggere gli option var..
'''

class SplitOptionsUI():
	def __init__(self):

		#Tooltips
		self.tooltipArray = {

		'Split1' : 'Split the selected mesh with one or more curves on mesh',
		'Split2' : 'Add points on splitted edge to increase curvature fidelity',
		'Split3' : 'The max points that will be added on the edge',
		'Split4' : 'Minimum distance in world units for adding points on the splitted edge.',

		'Corner Bias' : 'Consider a corner the angles over this degree',
		'Geo Bias' : 'Set to 0 to create new geometry with the split.\nSet to 1 to avoid small triangles and ngons, without creating new geometry\nIntermediate value will test if new geometry needs to be created instead of moving points.',		
		'Sections Spans' : 'Set the number of spans for the blending strips.'

		}

		#We define here the default values for the options
		self.hmOpt_defaultSeparate = 1
		self.hmOpt_defaultDelCurve = 0
		self.hmOpt_defaultKeepOrig = 0
		self.hmOpt_defaultGeoBias = 0.3
		self.hmOpt_defaultCornBias = 0.0
		self.hmOpt_defaultRefine = 0
		self.hmOpt_defaultMaxPt = 10
		self.hmOpt_defaultChordDist = 0.01




		#We store the values in these list with 2 entry: the first one is the string for the name, and the second one is the value
		self.hmOpt_SeparateVal = ['hmOpt_SeparateVal', self.hmOpt_defaultSeparate]
		self.hmOpt_DelCurveVal = ['hmOpt_DelCurveVal', self.hmOpt_defaultDelCurve]
		self.hmOpt_KeepOrigVal = ['hmOpt_KeepOrigVal', self.hmOpt_defaultKeepOrig]
		self.hmOpt_GeoBiasVal = ['hmOpt_GeoBiasVal', self.hmOpt_defaultGeoBias]
		self.hmOpt_CornBiasVal = ['hmOpt_CornBiasVal', self.hmOpt_defaultCornBias]
		self.hmOpt_RefineVal = ['hmOpt_RefineVal', self.hmOpt_defaultRefine]
		self.hmOpt_MaxPtVal = ['hmOpt_MaxPtVal', self.hmOpt_defaultMaxPt]
		self.hmOpt_ChordDistVal = ['hmOpt_ChordDistVal', self.hmOpt_defaultChordDist]


		#Define a list containing all the attributes
		self.attrList = [self.hmOpt_SeparateVal, self.hmOpt_DelCurveVal, self.hmOpt_KeepOrigVal, self.hmOpt_GeoBiasVal, self.hmOpt_CornBiasVal, self.hmOpt_RefineVal, self.hmOpt_MaxPtVal, self.hmOpt_ChordDistVal]


		#For every option we check if an option var is defined, if yes we sobstitute the value with the default ones
		uiUtil.optionVarInit( self.attrList )


	def createUiElements(self):
		'''
		This function implements the shared UI elements for mesh setup.
		We will use the same options both on the hmDockUi and from the floating panels options, for this reason we should avoid name conflicts of the widgets...
		'''
		self.cbSplitOpt = cmds.checkBoxGrp( numberOfCheckBoxes=3, labelArray3=['Separate', 'Delete Curves', 'Keep Original'], columnWidth = ([1, 75], [2,100], [3,90]), v1 = self.hmOpt_SeparateVal[1], v2 = self.hmOpt_DelCurveVal[1], v3 = self.hmOpt_KeepOrigVal[1] )
		cmds.separator( style='in', height = 1 )
		self.fsGeoBias = cmds.floatSliderGrp( label ='Geometry Bias', field=True, columnWidth= ([2, 50], [3,100]), value =self.hmOpt_GeoBiasVal[1], max = 1, min = 0, ann = self.tooltipArray['Geo Bias'])	
		self.fsCorBias = cmds.floatSliderGrp( label ='Corner Bias', field=True, columnWidth= ([2, 50], [3,100]), value = self.hmOpt_CornBiasVal[1], max = 180, min = 0, ann = self.tooltipArray['Corner Bias'])	
		cmds.separator( height=10, style='in' )
		self.cbRefEdge = cmds.checkBox( label = 'Refine Edge', value = self.hmOpt_RefineVal[1], onc = self.enableRefineOpt, ofc = self.disableRefineOpt, ann = self.tooltipArray['Split2'] )
		#cmds.rowLayout(nc = 2, rowAttach =[ (1, 'top', 5), (2, 'top', 5) ] )
		self.isRefMaxPt = cmds.intSliderGrp( label ='Max Points', field=True, columnWidth= ([2, 50], [3,100]), value = self.hmOpt_MaxPtVal[1], max = 100, min = 0, en=False, ann = self.tooltipArray['Split3'])	
		self.isAdaptVal = cmds.floatSliderGrp( label ='Chord Dist', field=True, columnWidth= ([2, 50], [3,100]), precision = 5, value = self.hmOpt_ChordDistVal[1], max = 1, min = 0, en=False, ann = self.tooltipArray['Split4'])	
		
		#Reset the parent for the next controls
		cmds.setParent( '..' )
		cmds.setParent( '..' )
		


	#================================ COMMANDS ========================================================
	def setOptions(self, *Args):

		#If refine is off..
		#print 'refine is true or false?', self.hmOpt_RefineVal[1]
		if self.hmOpt_RefineVal[1] == False:
			self.hmOpt_MaxPtVal[1] = 0
			self.hmOpt_ChordDistVal[1] = 1

		#===== Set Options
		cmds.hmSplitMeshByCurveCfg( ps = self.hmOpt_SeparateVal[1], dc= self.hmOpt_DelCurveVal[1], km= self.hmOpt_KeepOrigVal[1], gb= self.hmOpt_GeoBiasVal[1], cb= self.hmOpt_CornBiasVal[1],  es= self.hmOpt_MaxPtVal[1], et= self.hmOpt_ChordDistVal[1]  )




	def performCommand(self, *Args ):
		'''
		Perform the command
		'''		
		#Get the interface values
		self.getValuesFromInterface()	

		#Set the options into the mem handler
		self.setOptions()

		#==== Perform Solving
		cmds.hmSplitMeshByCurve()


	def saveOptionsValues(self):
		'''
		This method stores the current options in the optionVar 
		'''
		#Get the interface values
		self.getValuesFromInterface()
		#Save them
		uiUtil.optionVarSave( self.attrList )



	def resetOptionsValues(self):
		'''
		This method reset the options with the defualt values
		'''
		#Reset the ui values
		cmds.checkBoxGrp( self.cbSplitOpt, e=True, v1=self.hmOpt_defaultSeparate)
		cmds.checkBoxGrp( self.cbSplitOpt,  e=True, v2=self.hmOpt_defaultDelCurve)
		cmds.checkBoxGrp( self.cbSplitOpt,  e=True, v3=self.hmOpt_defaultKeepOrig)
		cmds.floatSliderGrp( self.fsGeoBias, e=True, v=self.hmOpt_defaultGeoBias )
		cmds.floatSliderGrp( self.fsCorBias, e=True, v=self.hmOpt_defaultCornBias )
		cmds.checkBox( self.cbRefEdge, e=True, v=self.hmOpt_defaultRefine )
		cmds.intSliderGrp( self.isRefMaxPt, e=True, v=self.hmOpt_defaultMaxPt )
		cmds.floatSliderGrp( self.isAdaptVal, e=True, v=self.hmOpt_defaultChordDist )



	#================================ INTERFACE UPDATES ========================================================
	def enableRefineOpt(self, *Args ):
		cmds.intSliderGrp( self.isRefMaxPt, edit=True, en=True )
		cmds.floatSliderGrp( self.isAdaptVal, edit=True, en=True ) 
	
	def disableRefineOpt(self, *Args ):
		cmds.intSliderGrp( self.isRefMaxPt, edit=True, en=False )
		cmds.floatSliderGrp( self.isAdaptVal, edit=True, en=False ) 


	#================================ UTILITY ========================================================
	def getValuesFromInterface(self, *Args):
		'''
		This method store in the main variables the values of the interface
		'''

		self.hmOpt_SeparateVal[1] = cmds.checkBoxGrp( self.cbSplitOpt, query=True, v1=True )
		self.hmOpt_DelCurveVal[1] = cmds.checkBoxGrp( self.cbSplitOpt,  query=True, v2=True )
		self.hmOpt_KeepOrigVal[1] = cmds.checkBoxGrp( self.cbSplitOpt,  query=True, v3=True)
		self.hmOpt_GeoBiasVal[1] = cmds.floatSliderGrp( self.fsGeoBias, query=True, v=True )
		self.hmOpt_CornBiasVal[1] = cmds.floatSliderGrp( self.fsCorBias, query=True, v=True )
		self.hmOpt_RefineVal[1] = cmds.checkBox( self.cbRefEdge, query=True, v=True )
		self.hmOpt_MaxPtVal[1] = cmds.intSliderGrp( self.isRefMaxPt, query=True, v=True )
		self.hmOpt_ChordDistVal[1] = cmds.floatSliderGrp( self.isAdaptVal, query=True, v=True )




	def getMainTooltip(self):
		'''
		This method return the main tooltip hint for adding it to the main button
		'''
		return self.tooltipArray['Split1']

