#!/usr/bin/env python
# -*- coding: utf-8 -*-

import maya.cmds as cmds




class HmSelectionHandling():
	'''
	This class implements some common methods for handling, validating selections.

	
	'''
	def __init__(self):
		#===== Prendiamo la selezione..
		self.initSel = cmds.ls( sl=True, long=True, fl = True)	
		
		#Validation parameters
		self.min = None
		self.max = None
		self.type = None
		self.extractType = None

		#Error messages
		self.errorIfZero = None
		self.errorIfSmallThanMin = None
		self.errorIfOutLimit =None
		
	def setFromList(self, inList):
		'''
		Set the selection to operate on from a list instead.
		'''
		self.initSel = inList


	def extract(self, selType):
		'''
		This commands extract the specified elements the list if present.
		If nothing is found nothing happens.
		This is useful for passing this extracted elements to a validate in a different moment.

		currently supported types:
		mesh
		nurbsCurve
		'''

		# Here we save the extracted elements
		extractedList = []
		objToKeep = []

		for object in self.initSel:

			# double check the transform and the shape for being sure to extract the right type
			shape = cmds.listRelatives( object, fullPath=True, shapes = True )
			# check the object
			if cmds.objectType( object, isType= selType ):
				extractedList.append(object)
			# check the shape
			elif shape != None:
				# print shape, selType
				if cmds.objectType( shape, isType= selType ):
					extractedList.append( object )
			# if nothing match, leave it to the main list
			else:
				objToKeep.append( object )

		# Save the object to keep in the class list
		self.initSel = objToKeep

		# return the found objects of matching type
		return extractedList

	def ignoreComponents(self):
		'''
		Ignore component selection by getting only the hilited objects
		'''
		componentSelection = self.initSel 

		self.initSel = cmds.ls(hl=True) #
		# self.initSel = cmds.ls( objectsOnly=True )

		return componentSelection


	def validate(self, selMin, selMax, selType):
		'''
		selMin: the minimum valid items, if None is ignored. when components are selected this is the number of objects
		selMax: the maximum valid items, if None is ignored. when components are selected this is the number of objects
		selType: the type of element valid for passing the test, if None is ignored
		
		currently supported types: 
		mesh


		edges
		'''
		
		#Set current data
		self.min = selMin
		self.max = selMax
		self.type = selType
		
		
		# if self.type == 'edges' or 'vertices' or 'faces':
		# 	# Save here the supposed components
		# 	self.componentSelection = self.initSel 
		# 	# store the objects
		# 	self.initSel = cmds.ls(hl=True)


		
		#===== Validiamo la selezione
		if len(self.initSel) == 0:
			if self.errorIfZero == None:
				cmds.error( 'Selection is empty' )
			else:
				cmds.error( self.errorIfZero )
				
				
		#Min checking, if required.
		if self.min != None:
			#If the selection is smaller
			if len(self.initSel) < self.min:
				#Display default or custom error
				if self.errorIfSmallThanMin == None:
					cmds.error( 'Selection is smaller than %s' % self.min )
				else:
					cmds.error( self.errorIfSmallThanMin )
			
			
		#Max checking, if required
		if self.max != None:
			#If the selection is bigger
			if len(self.initSel) > self.max:
				#Display default or custom error
				if self.errorIfOutLimit == None:
					cmds.error( 'Selection is bigger than %s' % self.max )
				else:
					cmds.error( self.errorIfOutLimit )
		

		#If no errors on selections, validate the type EXTEND THIS PART WITH ALL OR ANYWAY MORE SHAPES TYPES
		if self.type == 'mesh':
			for object in self.initSel:

				shape = cmds.listRelatives( object, fullPath=True, shapes = True ) [0]
				if cmds.objectType( shape, isType= self.type ) == 0:
					cmds.error( '%s invalid. This command works only with mesh' % shape )


				
		# if self.type == 'edge':
		# 	edges = cmds.filterExpand(self.componentSelection, sm=31)
		# 	print edges
		# 	if edges == None:
		# 		cmds.error( 'This command works only with poligons edges.' )
		# 	else:
		# 		self.initSel = edges



		return self.initSel





		
	def setErrorMessageIfZero(self, stringError):
		'''
		Set the error to display if the selection is empty
		'''
		self.errorIfZero = stringError
		
		
	def setErrorIfSmallThanMin(self, stringError):
		'''
		Set the error to display if the selection is smaller than minimum
		'''
		self.errorIfSmallThanMin = stringError	
		
		
	def setErrorMessageIfOutLimit(self, stringError):
		'''
		Set the error to display if the selection is bigger than maximum
		'''
		self.errorIfOutLimit = stringError
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	