#!/usr/bin/env python
# -*- coding: utf-8 -*-

import maya.cmds as cmds
import hmScripts.UI.hmUiCommonUtils as uiUtil
reload(uiUtil)

class SnapOptionsUI():
	def __init__(self):

		#Tooltips
		self.tooltipArray = {
		'Snap1' : 'Snap selected vertices to the nearest point of the selected curve'
		}

		#We define here the default values for the options
		self.hmOpt_defaultDelCurve = 1



		#We store the values in these list with 2 entry: the first one is the string for the name, and the second one is the value
		self.hmOpt_DelCurveVal = ['hmOpt_DelCurveVal', self.hmOpt_defaultDelCurve]


		self.optionsList = [self.hmOpt_DelCurveVal]

		# #For every option we check if an option var is defined, if yes we sobstitute the value with the default ones
		uiUtil.optionVarInit( self.optionsList )


	def createUiElements(self):
		'''
		This function implements the shared UI elements for mesh setup.
		We will use the same options both on the hmDockUi and from the floating panels options, for this reason we should avoid name conflicts of the widgets...
		'''
		cmds.separator( style='in', height = 1 )

		#================== Options
		self.cbSnapDelCrv = cmds.checkBox( label = 'Delete Curve', value = self.hmOpt_DelCurveVal[1] )
		#cmds.floatSliderGrp( 'fsRelaxAfterSnap', label ='Relax', field=True, width=1, columnWidth= ([2, 50], [3,100]), value = 30, max = 180, min = 0, en=False )	
		
		#Reset the parent for the next controls
		cmds.setParent( '..' )
		cmds.setParent( '..' )


	#================================ COMMANDS ========================================================

	def performCommand(self, *Args ):
		'''
		Perform the command
		'''		
		#Get the interface values
		self.getValuesFromInterface()
		
		#======== Perform command
		cmds.hmSnapVertexOnCurveCfg( dc= self.hmOpt_DelCurveVal[1], cb = 0 )
		cmds.hmSnapVertexOnCurve()

	def saveOptionsValues(self):
		'''
		This method stores the current options in the optionVar 
		'''
		#Get the interface values
		self.getValuesFromInterface()
		#Save them
		uiUtil.optionVarSave( self.optionsList )



	def resetOptionsValues(self):
		'''
		This method reset the options with the defualt values
		'''

		#Reset the ui values
		cmds.checkBoxGrp( self.cbTopoMerge, edit=True, v1=self.hmOpt_defaultFixTopo)




	#================================ INTERFACE UPDATES ========================================================



	#================================ UTILITY ========================================================
	def getValuesFromInterface(self, *Args):
		'''
		This method store in the main variables the values of the interface
		'''
		self.hmOpt_DelCurveVal[1] = cmds.checkBox( self.cbSnapDelCrv, query=True, v=True)

		


	def getMainTooltip(self):
		'''
		This method return the main tooltip hint for adding it to the main button
		'''
		return self.tooltipArray['Snap1']




