#!/usr/bin/env python
# -*- coding: utf-8 -*-


######################################
# This class creates the nodes connections for: hmOperations, hmIntersections, hmSplits, hmBlendings


import maya.cmds as cmds
import re

import hmScripts.UI.hmGlobalVariables as gVar
import hmScripts.Main.hmUtility as hmUtil
import hmScripts.UI.hmGradientControlFunctions as gradFunct
import hmScripts.UI.hmUiCommonUtils as uiUtils

reload(gVar)
reload(hmUtil)
reload(gradFunct)
reload(uiUtils)

class HmOperationConnections():
	'''
	This class create the connections for the hmCompound setup.
	Other than connecting nodes for the network creation it also handle the creation of the layers in the scene.
	'''
	###Setup
	def __init__(self): #We will have to add a shader option here..

			# Avoid the array connections.. this will avoid the nodes to be called too much
			self.noArray = True
			#Se vero, ogni volta crea dei layer nuovi...
			alwaysNewLayer = False
			
			#Options
			self.wireSourceMesh = True
			
			#Shader Name 
			self.hmShader = 'lambert1' #Default one

			
			#Connection attribute names
			self.previewAndNextCompound = gVar.previewAndNextCompound
			self.outMeshToCompound = gVar.outMeshToCompound #For original meshes
			
			self.inMeshForCompoundA = gVar.inMeshForCompoundA
			self.inMeshForCompoundB = gVar.inMeshForCompoundB
			self.outCompoundToMesh = gVar.outCompoundToMesh 
			
			self.inMeshFromCompound = gVar.inMeshFromCompound #For out meshes
			
			self.operationNodeAttr = gVar.operation #We connect our custom enum to this node parameter
			
			#Layers names
			self.origLayerName = gVar.origLayerName #Utility layer for hiding input mesh
			self.outLayerName = gVar.outLayerName #Utility layer for output meshes
			self.curvesLayerName = gVar.curvesLayerName #Utility layer for curves on meshes

			
					
			#General attr Names
			self.operationType = gVar.operationType
			self.smoothAttrName = gVar.smoothAttrName
			self.offsetAttrName = gVar.offsetAttrName
			
			#Hierarchy names
			self.networkName = 'hmNetwork' 
			self.intersectorsGrp = 'hmIntersectors' #transform for intersector nodes
			self.curvesGrp = 'hmCurves' #Stores the output of intersectors


			
			self.numericIncrement = 0
			
			
			
			#================  Hierarchy Handling
			#Verifichiamo quanti hmNetwork ci sono in scena..
			netList = cmds.ls('%s*' % self.networkName)
			netNum = ' '
			
			if len(netList) > 0:
				#Create an epty list for storing nmbers of hmNetworks found
				numberList = []
				
				#Now for every element search the numerical part..
				for el in netList:
					# print 'element:', el
					#If the network has a number inside...
					if hmUtil.hasNumbers(el):
						#Get the number as int, append to the current list and make sure that we save only the number without any list.
						numberList.append(  int(hmUtil.getNumbers(el)[0]) )
					#If is the first one, we will start adding 1..
					else:
						# print 'no number found..'
						netNum = 1 

						
				# if we find more network, we will have a list with numbers...
				if len(numberList)>= 1: 
					#Make sure that the list is ordered..		
					numberList.sort()
					#Reverse it
					numberList.reverse()
					#Now assign the right number for the creation of groups
					netNum = numberList[0] +1
				
				
			#Creiamo un hmNet
			newNetwork = cmds.createNode( 'transform', n= '%s%s' % (self.networkName, netNum) )
			
			self.networkName = '%s%s' % (self.networkName, netNum )
			self.intersectorsGrp = '%s%s' % (self.intersectorsGrp, netNum )						
			self.curvesGrp = '%s%s' % (self.curvesGrp, netNum )

			#Se ce ne sono, diamo un nome corretto alla variabile in modo da trovarlo correttamente dopo
			if netNum >= 1:			
			
				self.numericIncrement  = netNum
				
				if alwaysNewLayer:
					self.origLayerName = '%s%s' % (self.origLayerName, netNum )
					self.outLayerName = '%s%s' % (self.outLayerName, netNum )
					self.curvesLayerName = '%s%s' % (self.curvesLayerName, netNum )
					
					

			
			#================  Shader Handling
			#Get the shader from the optionVar
			self.controlShaderOptionVar()
			
			#Create it if necessary..
			self.createShader(self.hmShader)
			
			
			#Creiamo i gruppi
			cmds.createNode( 'transform', n= self.intersectorsGrp )
			cmds.createNode( 'transform', n= self.curvesGrp )
		
			#Mettiamoli in gerarchia
			cmds.parent( self.intersectorsGrp, self.curvesGrp, self.networkName ) 
			
			#========== End of def

			
				
				
				
	#############################################################################################################################################################################
	def tubeIntersection(self, meshA, meshB, radius):
		'''
		Test method for tubes intersections
		'''
		#mesh-mesh operations
		self.meshA = meshA
		self.meshB = meshB

		# Intersection curve
		self.createIntersectionNodes( self.meshA, 	[self.meshA, 'outMesh'], 		self.meshB,		[self.meshB, 'outMesh'], 	'intersection', showCrv=True)
	


	def intersectionCurvesFromMeshes(self, meshA, meshB, smoothValA, smoothValB, curveOperation, offsetVal, offsetType ):
		'''
		We create this simple network for having only intersections
		'''

		#mesh-mesh operations
		self.meshA = meshA
		self.meshB = meshB
		
		#Creiamo un parametro bloccato come separatore
		cmds.addAttr( self.networkName, longName='hmOp', at= 'enum',  en = 'Hard Mesh Op:', k=True)
		cmds.setAttr( '%s.hmOp' % self.networkName , lock= True )	
		
		#Add Operations Attr to the group network 
		cmds.addAttr( self.networkName, ln = self.operationType,  at = 'enum', en = '%s:%s:%s:%s:%s:%s' % (gVar.operationA, gVar.operationB, gVar.operationC, gVar.operationD, gVar.operationE, gVar.operationF), k=True )
		cmds.setAttr( '%s.%s' % (self.networkName, self.operationType), curveOperation-1 )	

		# ====================================================================================== 
		# ============================= HARDMESH OFFSETS =======================================
		# ======================================================================================
		if offsetType == 1:

			if smoothValA != 0 or smoothValB != 0:
				#=====================================
				#========== Layers Creation ==========	
				#=====================================
				self.createLayer( self.origLayerName, gVar.origLayerColor, 0 )
				cmds.editDisplayLayerMembers(self.origLayerName, [self.meshA, self.meshB] , noRecurse=True )
				
				# ================ MAKE WIREFRAME ==================
				uiUtils.makeItWireframe( self.meshB )
				uiUtils.makeItWireframe( self.meshA )

				# =================== NODE =====================
				self.offsetNode = cmds.createNode('hmOffsetCurveNode') #Create an hmOffsetCurve Node

				smoothedA = self.createSmoothNode( self.meshA, smoothValA, mode = 1 )
				smoothedB = self.createSmoothNode( self.meshB, smoothValB, mode = 1 )

				# #Create the offset attribute on the network and connect it to the offset node attribute
				self.addAndConnectAttr( self.networkName, self.offsetAttrName , '%s.offsetA' % self.offsetNode, offsetVal, 'double', minVal = 0 )
				self.addAndConnectAttr( self.networkName, self.offsetAttrName , '%s.offsetB' % self.offsetNode, offsetVal, 'double', minVal = 0 )
				
				#===========================================================
				#================ MESH TO ORDER AND TO COMPOUND CONNECTIONS
				#===========================================================
				self.createMeshToHmOffsetConnections( self.meshA, smoothedA, 0 )
				self.createMeshToHmOffsetConnections( self.meshB, smoothedB, 1 )
				
					
				#===========================================
				#================ CREATE INTERSECTORS	
				#===========================================
				# self.createIntersectionNodes( self.meshA, 	[self.meshA, 'outMesh'], 		self.meshB,		[self.meshB, 'outMesh'], 	'intersection', showCrv=True)	# This bypass the smooth....
				self.createIntersectionNodes( self.meshA, 	smoothedA, 		self.meshB,		smoothedB, 	'intersection')
				
				
				#===========================================================
				#================ OFFSET AND COMPOUND CONNECTIONS ==========
				#===========================================================
				self.createCurvesConnectionHmOffset( compoundExist = False )


				#================  Shader Handling
				#Get the shader from the optionVar
				self.controlShaderOptionVar()
				
				#Create it if necessary..
				self.createShader(self.hmShader)

				#================ SHADER ASSIGNMENT
				cmds.select( meshA, meshB, r=True )
				cmds.hyperShade( assign = self.hmShader )
				cmds.select(cl=True)


			else:
				# Create the nodes
				self.offsetNode = cmds.createNode('hmOffsetCurveNode') #Create an hmOffsetCurve Node
				self.createIntersectionNodes( self.meshA, 	[self.meshA, 'outMesh'], 		self.meshB,		[self.meshB, 'outMesh'], 	'intersection', showCrv=True)	

				# Connect the mesh to the hm offset node
				cmds.connectAttr('%s.outMesh' % self.meshA, '%s.%s[0]' % (self.offsetNode, gVar.inMeshes) )		
				cmds.connectAttr('%s.worldMatrix[0]' % self.meshA, '%s.%s[0]' % (self.offsetNode, gVar.inMeshesMatrix) )		
			
				cmds.connectAttr('%s.outMesh' % self.meshB, '%s.%s[1]' % (self.offsetNode, gVar.inMeshes) )		
				cmds.connectAttr('%s.worldMatrix[0]' % self.meshB, '%s.%s[1]' % (self.offsetNode, gVar.inMeshesMatrix) )	
				
				#Create the offset attribute on the network and connect it to the offset node attribute
				self.addAndConnectAttr( self.networkName, self.offsetAttrName , '%s.offsetA' % self.offsetNode, offsetVal, 'double', minVal = 0 )
				self.addAndConnectAttr( self.networkName, self.offsetAttrName , '%s.offsetB' % self.offsetNode, offsetVal, 'double', minVal = 0 )
				# cmds.connectAttr( '%s.outputX' % multNode, '%s.localTranslateZ' % offsetNode )
				
				# Create the connections for the curves
				self.createCurvesConnectionHmOffset( compoundExist = False )
				
				
		# ====================================================================================== 
		# ================================ MAYA OFFSETS =======================================
		# ======================================================================================
		# Very complicated node network created for using maya nodes in the creation of OFFSETS
		if offsetType == 2:
			
			#=====================================
			#=== Direction Choice Attributes =====	
			#=====================================
			self.createDirectionChoicesParameters( self.networkName )
			
			
			#===========================================
			#================ MESH OFFSETS
			#===========================================	
			offsetA = self.createMeshOffset( self.meshA, [self.meshA, 'outMesh'], offsetVal, self.networkName )
			offsetB = self.createMeshOffset(self.meshB, [self.meshB, 'outMesh'], offsetVal, self.networkName )
			
			
			#===========================================
			#================ CREATE DIRECTION CHOICE		
			#===========================================	
			multA = offsetA[1]
			multB = offsetB[1]
			choiceA = self.createDirectionChoice( self.unionVal, self.diffAVal, self.diffAVal1, self.intVal, self.panelAVal, self.panelAVal1, '%s.input2X' % multA, self.networkName ) #A e b sono diversi per il difference...
			choiceB = self.createDirectionChoice( self.unionVal, self.diffBVal, self.diffBVal1, self.intVal, self.panelBVal, self.panelBVal1, '%s.input2X' % multB, self.networkName )
			
			
			#===========================================
			#================ CREATE PANEL CHOICE		
			#===========================================
			panelChoiceA = self.createPanelChoice( [self.meshA, 'outMesh'], [offsetA[2], 'output'], [offsetB[0], 'output'], [self.meshA, 'outMesh'] )
			panelChoiceB = self.createPanelChoice( [offsetA[0], 'output'],[self.meshA, 'outMesh'], [self.meshB, 'outMesh'], [offsetB[2], 'output'] )

			
			#Create and connect 2 condition nodes
			conditionA = cmds.createNode('condition')
			cmds.setAttr('%s.secondTerm' % conditionA, 4)
			cmds.setAttr('%s.operation' % conditionA, 1) #Set to noe equal
			cmds.connectAttr('%s.%s' % (self.networkName, self.operationType), '%s.firstTerm' % conditionA )
			cmds.connectAttr('%s.outColor.outColorR' % conditionA, '%s.selector' % panelChoiceA[0]  )
			
			conditionB = cmds.createNode('condition')
			cmds.setAttr('%s.secondTerm' % conditionB, 5)
			cmds.setAttr('%s.operation' % conditionB, 1) #Set to noe equal
			cmds.connectAttr('%s.%s' % (self.networkName, self.operationType), '%s.firstTerm' % conditionB )
			cmds.connectAttr('%s.outColor.outColorR' % conditionB, '%s.selector' % panelChoiceB[0]  )
			
			#===========================================
			#================ CREATE INTERSECTORS	
			#===========================================
			#Panel choice
			self.createIntersectionNodes( self.meshA, 	[panelChoiceA[0], 'output'], 		self.meshB, 	[panelChoiceA[1], 'output'], 'offsetA', showCrv=True)
			self.createIntersectionNodes( self.meshA, 	[panelChoiceB[0], 'output'], 	self.meshB, 	[panelChoiceB[1], 'output'], 'offsetB', showCrv=True)	
			self.createIntersectionNodes( self.meshA, 	[self.meshA, 'outMesh'], 		self.meshB,		[self.meshB, 'outMesh'], 	'intersection', showCrv=True)		
		

		
		
	def hmCompoundSetup(	self, 	meshA, meshB, curveOperation, smoothValA, smoothValB, offsetValueA, offsetValueB, cornerBiasVal, geoBiasVal, 
									sectVal, refEdgeVal, chordRefEdge, blendStrtVal, interBendVal, weightAVal, weightBVal, useRampVal, rampValues, 
									rampChordDist, rampOffsetVal, rampScaleVal, rampModeVal, bindBiasVal, removeCurlVal, compNormVal, offsetType, scaleFactor
						):
		'''
		This method will create the basic elements for the network creation.
		'''
		
		# ====================================================================================== 
		# ============================= HARDMESH OFFSETS =======================================
		# ======================================================================================
		if offsetType == 1:

			#=====================================
			#======== IN Mesh Preparation ==========	
			#=====================================
			# Store the input mesh in two vars
			self.meshA = meshA
			self.meshB = meshB


			if self.wireSourceMesh:		
				#Remove shaders and assign color
				uiUtils.makeItWireframe( self.meshA )


				# ========================================================
				# ================ Multi mesh support ====================
				if gVar.multimesh:
					for el in self.meshB:
						uiUtils.makeItWireframe( el )
				else:
					uiUtils.makeItWireframe( self.meshB )


			self.originalMeshForLayer = [self.meshA, self.meshB] #We use this list for the add to the layer


			# ========================================================
			# ================ Multi mesh support ====================
			if gVar.multimesh:
				# If we have more than one mesh
				if len(meshB) > 1:
					# Combine the meshes and return the original one
					self.meshB = uiUtils.smartCombine(self.meshB) [0]
				else:
					self.meshB = self.meshB[0]
			


			
			#=====================================
			#========== Layers Creation ==========	
			#=====================================
			self.createLayer( self.origLayerName, gVar.origLayerColor, 0 )
			self.createLayer( self.outLayerName, gVar.outLayerColor, 2 )
			self.createLayer( self.curvesLayerName, gVar.curvesLayerColor, 0 )
			
			
			#=====================================
			#========== CREATE HM NODES ============	
			#=====================================	
			self.offsetNode = cmds.createNode('hmOffsetCurveNode') #Create an hmOffsetCurve Node
			self.compoundNode = cmds.createNode('hmCompoundNode', name='hmCompound') #Create an hmCompoundNode
			
			#Add the connection attributes, we will use them just for quering the connections for the smart edit window
			cmds.addAttr( self.compoundNode, ln = self.previewAndNextCompound,  at = 'bool' )
			cmds.addAttr( self.compoundNode, ln = self.inMeshForCompoundA,  at = 'bool' )
			cmds.addAttr( self.compoundNode, ln = self.inMeshForCompoundB,  at = 'bool' )
			cmds.addAttr( self.compoundNode, ln = self.outCompoundToMesh,  at = 'bool' )


			# We add this attribute for being able to recognize when the split refine has been set on scale
			# cmds.addAttr( self.compoundNode, ln = gVar.edgeSubdD,  at = 'bool' )
			
			
			#=====================================
			#======== Out Mesh Creation ==========	
			#=====================================
			self.resultingMesh = cmds.createNode('mesh', n= gVar.intermediateMeshName )
			transform = cmds.listRelatives( self.resultingMesh, parent = True ) [0]

			# Connecting to the transform cause maya freeze when updating smooth levels
			cmds.addAttr( transform, ln = self.inMeshFromCompound,  at = 'bool' )

			# cmds.connectAttr('%s.%s' %(self.compoundNode, self.outCompoundToMesh), '%s.%s' %(transform, self.inMeshFromCompound))

			# We connect to the shape since the transform was causing problems
			cmds.addAttr( self.resultingMesh, ln = self.inMeshFromCompound,  at = 'bool' )
			cmds.connectAttr('%s.%s' %(self.compoundNode, self.outCompoundToMesh), '%s.%s' %(self.resultingMesh, self.inMeshFromCompound))
			

			# ===================================================================================
			# ===================================================================================
			# This is the old attributes connection and creation.. grown in the developing phase

			#Add Operations Attr to the compound node 
			# cmds.addAttr( self.compoundNode, ln = self.operationType,  at = 'enum', en = '%s:%s:%s:%s:%s:%s' % (gVar.operationA, gVar.operationB, gVar.operationC, gVar.operationD, gVar.operationE, gVar.operationF), k=True )
			# cmds.setAttr( '%s.%s' % (self.compoundNode, self.operationType), curveOperation-1 )
			
			#Connect this attribute to the node one
			# cmds.connectAttr('%s.%s' % (self.compoundNode, self.operationType), '%s.%s' % (self.compoundNode, self.operationNodeAttr) )

			# And to the offset node (that is the real one controlling the operation type)
			# cmds.connectAttr('%s.%s' % (self.compoundNode, self.operationType), '%s.%s' % (self.offsetNode, gVar.booleanOperation) )
			
			#Create the offset parameters on the compound and connect them to the offset node
			# self.addAndConnectAttr( self.compoundNode, self.offsetAttrName , '%s.%s' % (self.offsetNode, gVar.offsetA), offsetValueA, 'double', minVal = 0 )
			# self.addAndConnectAttr( self.compoundNode, self.offsetAttrName , '%s.%s' % (self.offsetNode, gVar.offsetB), offsetValueB, 'double', minVal = 0 )


			#=====================================
			#====== Attributes connection ========	
			#=====================================

			#Add the connection attributes to the meshes, this will add it to the TRANSFORM.. keep it in mind
			self.addAndConnectAttr( self.meshA, self.outMeshToCompound, '%s.%s' % (self.compoundNode,self.inMeshForCompoundA), 0, 'bool', addPolicy = 'skipIfExist', keyable = False )
			self.addAndConnectAttr( self.meshB, self.outMeshToCompound, '%s.%s' % (self.compoundNode,self.inMeshForCompoundB), 0, 'bool', addPolicy = 'skipIfExist', keyable = False )
			

			# self.addAndConnectAttr( cmds.listRelatives(self.meshA, shapes=True)[0], self.outMeshToCompound, '%s.%s' % (self.compoundNode,self.inMeshForCompoundA), 0, 'bool', addPolicy = 'skipIfExist', keyable = False )
			# self.addAndConnectAttr( cmds.listRelatives(self.meshB, shapes=True)[0], self.outMeshToCompound, '%s.%s' % (self.compoundNode,self.inMeshForCompoundB), 0, 'bool', addPolicy = 'skipIfExist', keyable = False )
			
			# Connect the operation value of the offset to the operation value of the compount
			cmds.connectAttr('%s.%s' % (self.offsetNode, gVar.booleanOperation), '%s.%s' % (self.compoundNode, gVar.operation ) )
			
			# And set the value from the interface
			cmds.setAttr( '%s.%s' % (self.offsetNode, gVar.booleanOperation), curveOperation-1 )
			

			# Connect the two offset values for having only one radius
			cmds.connectAttr('%s.%s' % (self.offsetNode, gVar.offsetA) , '%s.%s' % (self.offsetNode, gVar.offsetB) )
	
			


			#=====================================
			#========= Set basic attr ============	
			#=====================================	
			# == Compound attr to override offset ones
			# cmds.setAttr('%s.%s' % (self.compoundNode, gVar.offsetA),  offsetValueA )
			# cmds.setAttr('%s.%s' % (self.compoundNode, gVar.offsetB),  offsetValueB )

			# Set attributes
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.geometryBias),  geoBiasVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.sectionSpans),  sectVal )



			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.edgeSubd),  refEdgeVal )


			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.tolerance),  chordRefEdge )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.smoothFiller),  interBendVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.weightA),  weightAVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.weightB),  weightBVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.activeRampedFiller),  useRampVal )
			

			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.rampOffset),  rampOffsetVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.rampScale),  rampScaleVal )
			
			# == Offset
			cmds.setAttr('%s.%s' % (self.offsetNode, gVar.offsetA),  offsetValueA )

			if gVar.scaleFactor == 1:
				# Force quality
				cmds.setAttr('%s.%s' % (self.offsetNode, gVar.scaleFactor ),  scaleFactor )
				
				

			# ================== DEVELOP =================================
			# If we are in develop mode, set extra attributes
			if gVar.develop == 1:
				# cmds.setAttr('%s.%s' % (self.compoundNode, gVar.rampMode),  rampModeVal )
				cmds.setAttr('%s.%s' % (self.compoundNode, gVar.rampChordDist),  rampChordDist )
				cmds.setAttr('%s.%s' % (self.compoundNode, gVar.fillStrategy),  blendStrtVal )
				cmds.setAttr('%s.%s' % (self.compoundNode, gVar.cornerBias),  cornerBiasVal )
				# cmds.setAttr('%s.%s' % (self.offsetNode, gVar.removeCurl),  removeCurlVal )
				cmds.setAttr('%s.%s' % (self.offsetNode, 'quality'),  2 )
				cmds.setAttr('%s.%s' % (self.compoundNode, gVar.computeNormals),  compNormVal )



			
			#=====================================
			#========== Set Ramp attr ============	
			#=====================================	
			#For setting the ramp, we'll need to use a shared procedure..
			# cmds.setAttr('%s.%s' % (self.compoundNode, gVar.rampedFiller),  rampValues )
			# print 'NODE CREATION FOR RAMP, ricevo questi valori: ', rampValues
			if rampValues:
				gradFunct.changeGradientControlShape( self.compoundNode, gVar.rampBaseId, gVar.rampVal, gVar.rampPos, gVar.rampInt, rampValues)
			
			
			
			
						
			#=====================================
			#===== Mesh From Compound Query ======
			#=====================================
			#Mesh Query. Here we check if one of the provided mesh is already part of an hm network.
			#If it's the result of an operation, we avoid the smooth operation propting a warning.
			#If it's the result of an operation, we will connect also the compounds through the same attribute
			#for letting the ui having a list of all the involved compounds...

			isFromNetworkA = self.checkIfCameFromNetwork( self.meshA )
			isFromNetworkB = self.checkIfCameFromNetwork( self.meshB )
			
			if isFromNetworkA == True:
				smoothValA = 0
			if isFromNetworkB == True:
				smoothValB = 0


			#===========================================
			#================ INTERMEDIATE SMOOTH NODES	
			#===========================================
			smoothedA = self.createSmoothNode( self.meshA, smoothValA, mode = 0 )
			smoothedB = self.createSmoothNode( self.meshB, smoothValB, mode = 0 )
			
			

			
			#===========================================================
			#================ MESH TO ORDER AND TO COMPOUND CONNECTIONS
			#===========================================================
			self.createMeshToHmOffsetConnections( self.meshA, smoothedA, 0 )
			self.createMeshToHmOffsetConnections( self.meshB, smoothedB, 1 )
			
			

				
			#===========================================
			#================ CREATE INTERSECTORS	
			#===========================================
			# self.createIntersectionNodes( self.meshA, 	[self.meshA, 'outMesh'], 		self.meshB,		[self.meshB, 'outMesh'], 	'intersection', showCrv=True)	# This bypass the smooth....
			self.createIntersectionNodes( self.meshA, 	smoothedA, 		self.meshB,		smoothedB, 	'intersection')
			
			
			#===========================================================
			#================ OFFSET AND COMPOUND CONNECTIONS ==========
			#===========================================================
			self.createCurvesConnectionHmOffset()
			
			# ===================== Direct connection of arrays =====================
			
			#hmOffset to compound



			# We connect always meshes directly
			cmds.connectAttr( (self.offsetNode + '.' + gVar.outMeshes + '[0]' ), (self.compoundNode + '.' + gVar.inMeshes + '[0]' ) )
			cmds.connectAttr( (self.offsetNode + '.' + gVar.outMeshes + '[1]' ), (self.compoundNode + '.' + gVar.inMeshes + '[1]' ) )
			# cmds.connectAttr( (self.offsetNode + '.' + gVar.outMeshesMatrix ), (self.compoundNode + '.' + gVar.inMeshesMatrix ) )						
			
			
			#===========================================
			#================ LAYERS ===================
			#===========================================
			

			#Add the curves gourp to the layer
			cmds.editDisplayLayerMembers(self.curvesLayerName, self.curvesGrp, noRecurse=True )
				

			# Add the involved meshes to the original mesh layer..
			if gVar.multimesh:
				# print 'FOR LAYERS', self.originalMeshForLayer
				cmds.editDisplayLayerMembers(self.origLayerName, self.originalMeshForLayer[0] , noRecurse=True )
				cmds.editDisplayLayerMembers(self.origLayerName, self.originalMeshForLayer[1] , noRecurse=True )

			else:
				cmds.editDisplayLayerMembers(self.origLayerName, self.originalMeshForLayer , noRecurse=True )
			
			#This function was responsable for causing the undo problems, we will use another one that instead of connecting
			# the color from the layer will just read the color and set it on the shapes color
			# self.connecColorToShape( self.origLayerName, self.meshA )
			# self.connecColorToShape( self.origLayerName, self.meshB )

			# We are bypassing the color assignment
			'''
			if isFromNetworkA == False:
				self.getColorFromLayer( self.origLayerName, self.meshA  )
			if isFromNetworkB == False:
				self.getColorFromLayer( self.origLayerName, self.meshB  )
			'''
			
				
			
			#================ SHADER ASSIGNMENT
			cmds.select( self.resultingMesh, r=True )
			cmds.hyperShade( assign = self.hmShader )
			cmds.select(cl=True)

				
			
			#Connect the compound to the out mesh
			cmds.connectAttr( '%s.outputMesh' % self.compoundNode, '%s.inMesh' % self.resultingMesh)


			
			
			#================ LAYERS
			cmds.editDisplayLayerMembers(self.outLayerName, self.resultingMesh, noRecurse=True )
			
			
			# == Give a name to the out mesh
			newName = cmds.rename(transform, 'hmOutMesh')
			cmds.rename(self.resultingMesh, newName + 'Shape')
			
			
			
		# ====================================================================================== 
		# ================================ MAYA OFFSETS =======================================
		# ======================================================================================
		# Very complicated node network created for using maya nodes in the creation of OFFSETS
		if offsetType == 2:			
			
			
			
					
			#mesh-mesh operations
			self.meshA = meshA
			self.meshB = meshB
			
			self.originalMeshForLayer = [self.meshA, self.meshB] #We use this list for the add to the layer
			
			#Create an hmOrderToSplit Node
			self.orderToSplit = cmds.createNode('hmOrderToSplitNode')
			#Set the curve sequencer on
			# cmds.setAttr('%s.curveSequencer' % self.orderToSplit, 1)
			
			
			#Create an hmCompoundNode
			self.compoundNode = cmds.createNode('hmCompoundNode', name='hmCompound')
			
			
			#Add the connection attributes, we will use them just for quering the connections for the smart edit window
			cmds.addAttr( self.compoundNode, ln = self.previewAndNextCompound,  at = 'bool' )
			cmds.addAttr( self.compoundNode, ln = self.inMeshForCompoundA,  at = 'bool' )
			cmds.addAttr( self.compoundNode, ln = self.inMeshForCompoundB,  at = 'bool' )
			cmds.addAttr( self.compoundNode, ln = self.outCompoundToMesh,  at = 'bool' )
			
			
			#=====================================
			#===== Mesh From Compound Query ======
			#=====================================
			#Mesh Query. Here we check if one of the provided mesh is already part of an hm network.
			#If it's the result of an operation, we avoid the smooth operation propting a warning.
			#If it's the result of an operation, we will connect also the compounds through the same attribute
			#for letting the ui having a list of all the involved compounds...

			isFromNetworkA = self.checkIfCameFromNetwork( self.meshA )
			isFromNetworkB = self.checkIfCameFromNetwork( self.meshB )
			
			if isFromNetworkA == True:
				smoothValA = 0
			if isFromNetworkB == True:
				smoothValB = 0

					
			
			
			#Add the connection attributes to the meshes, this will add it to the TRANSFORM.. keep it in mind
			self.addAndConnectAttr( self.meshA, self.outMeshToCompound, '%s.%s' % (self.compoundNode,self.inMeshForCompoundA), 0, 'bool', addPolicy = 'skipIfExist', keyable = False )
			self.addAndConnectAttr( self.meshB, self.outMeshToCompound, '%s.%s' % (self.compoundNode,self.inMeshForCompoundB), 0, 'bool', addPolicy = 'skipIfExist', keyable = False )
			
			#Add Operations Attr to the compound node 
			cmds.addAttr( self.compoundNode, ln = self.operationType,  at = 'enum', en = '%s:%s:%s:%s:%s:%s' % (gVar.operationA, gVar.operationB, gVar.operationC, gVar.operationD, gVar.operationE, gVar.operationF), k=True )
			cmds.setAttr( '%s.%s' % (self.compoundNode, self.operationType), curveOperation-1 )
			
			#Connect this attribute to the node one
			cmds.connectAttr('%s.%s' % (self.compoundNode, self.operationType), '%s.%s' % (self.compoundNode, self.operationNodeAttr) )
			
			
			
			#=====================================
			#=== Direction Choice Attributes =====	
			#=====================================
			self.createDirectionChoicesParameters( self.compoundNode )

			
			
			#=====================================
			#========== Layers Creation ==========	
			#=====================================
			self.createLayer( self.origLayerName, gVar.origLayerColor, 0 )
			self.createLayer( self.outLayerName, gVar.outLayerColor, 2 )
			self.createLayer( self.curvesLayerName, gVar.curvesLayerColor, 0 )
			
			
				
			#=====================================
			#========= Set basic attr ============	
			#=====================================	
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.cornerBias),  cornerBiasVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.geometryBias),  geoBiasVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.sectionSpans),  sectVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.edgeSubd),  refEdgeVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.tolerance),  chordRefEdge )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.fillStrategy),  blendStrtVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.smoothFiller),  interBendVal )
			# cmds.setAttr('%s.%s' % (self.compoundNode, gVar.refinement),  refFillerVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.weightA),  weightAVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.weightB),  weightBVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.activeRampedFiller),  useRampVal )
			

			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.rampChordDist),  rampChordDist )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.rampOffset),  rampOffsetVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.rampScale),  rampScaleVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.rampMode),  rampModeVal )
			
			# == Order To Split 
			cmds.setAttr('%s.%s' % (self.orderToSplit, gVar.bindBias),  bindBiasVal )
			cmds.setAttr('%s.%s' % (self.orderToSplit, gVar.removeCurl),  removeCurlVal )
			# cmds.setAttr('%s.%s' % (self.orderToSplit, gVar.curveSequencer),  sequencerVal )
			# cmds.setAttr('%s.%s' % (self.orderToSplit, gVar.sequenceBias),  sequencerToleranceVal )
			
			# == Again compound	
			# cmds.setAttr('%s.%s' % (self.compoundNode, gVar.autoMergeResult),  autoMergeVal )
			# cmds.setAttr('%s.%s' % (self.compoundNode, gVar.mergeDist),  mergeTolVal )
			cmds.setAttr('%s.%s' % (self.compoundNode, gVar.computeNormals),  compNormVal )
			
			
			
			
			
			#=====================================
			#========== Set Ramp attr ============	
			#=====================================	
			#For setting the ramp, we'll need to use a shared procedure..
			# cmds.setAttr('%s.%s' % (self.compoundNode, gVar.rampedFiller),  rampValues )
			# print 'NODE CREATION FOR RAMP, ricevo questi valori: ', rampValues
			gradFunct.changeGradientControlShape( self.compoundNode, gVar.rampBaseId, gVar.rampVal, gVar.rampPos, gVar.rampInt, rampValues)

			
				
			#=====================================
			#======== Out Mesh Creation ==========	
			#=====================================
			self.resultingMesh = cmds.createNode('mesh')
			transform = cmds.listRelatives( self.resultingMesh, parent = True ) [0]
			cmds.addAttr( transform, ln = self.inMeshFromCompound,  at = 'bool' )
			cmds.connectAttr('%s.%s' %(self.compoundNode, self.outCompoundToMesh), '%s.%s' %(transform, self.inMeshFromCompound))
			
			
			
			#=====================================
			#================ IF MAKE IT WIREFRAME
			#=====================================
			
			if self.wireSourceMesh:		
				#Remove shaders and assign color
				uiUtils.makeItWireframe( self.meshA )
				uiUtils.makeItWireframe( self.meshB )
				
				
				
			
			
			
			
			
			#===========================================
			#================ INTERMEDIATE SMOOTH NODES	
			#===========================================
			smoothedA = self.createSmoothNode( self.meshA, smoothValA, mode = 0 )
			smoothedB = self.createSmoothNode( self.meshB, smoothValB, mode = 0 )
			
			# This solution has been removed... was causing a strange slowdown.. we will use a standard 
			# "mesh-smooth-mesh" setup for inserting the smooth in between
			#Gli smooth, che suddividono la geometria per passarla in uscita
			# smoothA = self.createSmoothNode( self.meshA, smoothValA, mode = 1 )
			# smoothB = self.createSmoothNode( self.meshB, smoothValB, mode = 1 
			

			
			#===========================================
			#================ MESH TO ORDER AND TO COMPOUND CONNECTIONS
			#===========================================	
			self.createMeshToCompoundConnections( self.meshA, smoothedA, 0 )
			self.createMeshToCompoundConnections( self.meshB, smoothedB, 1 )



			#===========================================
			#================ MESH OFFSETS
			#===========================================	
			offsetA = self.createMeshOffset( self.meshA, smoothedA, offsetValueA, self.compoundNode )
			offsetB = self.createMeshOffset(self.meshB, smoothedB, offsetValueB, self.compoundNode )
			

			
			
			#===========================================
			#================ CREATE DIRECTION CHOICE		
			#===========================================	
			multA = offsetA[1]
			multB = offsetB[1]
			choiceA = self.createDirectionChoice( self.unionVal, self.diffAVal, self.diffAVal1, self.intVal, self.panelAVal, self.panelAVal1, '%s.input2X' % multA, self.compoundNode ) #A e b sono diversi per il difference...
			choiceB = self.createDirectionChoice( self.unionVal, self.diffBVal, self.diffBVal1, self.intVal, self.panelBVal, self.panelBVal1, '%s.input2X' % multB, self.compoundNode )
			
			
			#===========================================
			#================ CREATE PANEL CHOICE		
			#===========================================
			panelChoiceA = self.createPanelChoice( smoothedA, [offsetA[2], 'output'], [offsetB[0], 'output'], smoothedB )
			panelChoiceB = self.createPanelChoice( [offsetA[0], 'output'],smoothedA, smoothedB, [offsetB[2], 'output'] )

			
			#Create and connect 2 condition nodes
			conditionA = cmds.createNode('condition')
			cmds.setAttr('%s.secondTerm' % conditionA, 4)
			cmds.setAttr('%s.operation' % conditionA, 1) #Set to noe equal
			cmds.connectAttr('%s.%s' % (self.compoundNode, self.operationType), '%s.firstTerm' % conditionA )
			cmds.connectAttr('%s.outColor.outColorR' % conditionA, '%s.selector' % panelChoiceA[0]  )
			
			conditionB = cmds.createNode('condition')
			cmds.setAttr('%s.secondTerm' % conditionB, 5)
			cmds.setAttr('%s.operation' % conditionB, 1) #Set to noe equal
			cmds.connectAttr('%s.%s' % (self.compoundNode, self.operationType), '%s.firstTerm' % conditionB )
			cmds.connectAttr('%s.outColor.outColorR' % conditionB, '%s.selector' % panelChoiceB[0]  )
			
			#===========================================
			#================ CREATE INTERSECTORS	
			#===========================================
			#Panel choice
			self.offsetANode = self.createIntersectionNodes( self.meshA, 	[panelChoiceA[0], 'output'], 		self.meshB, 	[panelChoiceA[1], 'output'], 'offsetA')
			self.offsetBNode = self.createIntersectionNodes( self.meshA, 	[panelChoiceB[0], 'output'], 	self.meshB, 	[panelChoiceB[1], 'output'], 'offsetB')	
			self.intersectionNode = self.createIntersectionNodes( self.meshA, 	smoothedA, 		self.meshB,		smoothedB, 	'intersection')
			
			
			
			#===========================================
			#================ CURVES CONNECTION		
			#===========================================
			
			# This method use the "old" dirent curve connection
			self.createCurvesConnection()
			
			# This method connect directly arrays, so the number of intersections can vary
			# self.createDirectCurvesConnection()
			
			
			
			
			#===========================================
			#================ LAYERS
			#===========================================
			

			#Add the curves gourp to the layer
			cmds.editDisplayLayerMembers(self.curvesLayerName, self.curvesGrp, noRecurse=True )
				

			# Add the involved meshes to the original mesh layer..
			cmds.editDisplayLayerMembers(self.origLayerName, self.originalMeshForLayer , noRecurse=True )
			
			#This function was responsable for causing the undo problems, we will use another one that instead of connecting
			# the color from the layer will just read the color and set it on the shapes color
			# self.connecColorToShape( self.origLayerName, self.meshA )
			# self.connecColorToShape( self.origLayerName, self.meshB )
			if isFromNetworkA == False:
				self.getColorFromLayer( self.origLayerName, self.meshA  )
			if isFromNetworkB == False:
				self.getColorFromLayer( self.origLayerName, self.meshB  )
			
				
			
			#================ SHADER ASSIGNMENT
			cmds.select( self.resultingMesh, r=True )
			cmds.hyperShade( assign = self.hmShader )
			cmds.select(cl=True)

				
			
			#Connect the compound to the out mesh
			cmds.connectAttr( '%s.outputMesh' % self.compoundNode, '%s.inMesh' % self.resultingMesh)


			
			
			#================ LAYERS
			cmds.editDisplayLayerMembers(self.outLayerName, self.resultingMesh, noRecurse=True )
			
			
			# == Give a name to the out mesh
			cmds.rename(transform, 'hmOutMesh')
			
			# print 'New HardMesh Operation succesfully created.'
		

		
		
		

	#############################################################################################################################################################################
	###################################################            SMALLER METHODS          ###############################################################################		
	#############################################################################################################################################################################	
	
	
	
	
	
	
	
	
	
	def checkIfCameFromNetwork(self, inputMesh ):
		'''
		It's used on the resulting meshes of others networks.
		We lock the smooth, remove from the original mesh layer and create useful connections for the smartEdit.
		In this way we will avoid to smooth mesh coming from an hm network in an automatic way, and we keep track 
		of the connection cascade creating a new connection with the current compound.
		'''

		
		meshType = uiUtils.queryMeshType( inputMesh )
		
		# print inputMesh, 'meshType is: ', meshType
		
		if meshType[0] == True:
			#If the mesh type is "in".. means that its the out of an existing hmCompound
			if meshType[1] == 'in' or meshType[1] == 'multi':
				#This just means that we found an hm attribute.. doesn't mean nothing more.
				

				# OLD TRANSFORM QUERY
				#Check if we have connections....
				# connections = cmds.listConnections('%s.%s' % (inputMesh, gVar.inMeshFromCompound ), d=True)

				# NEW SHAPE QUERY
				#Check if we have connections....
				shape = shape = cmds.listRelatives(inputMesh, shapes=True)[0]
				connections = cmds.listConnections('%s.%s' % (shape, gVar.inMeshFromCompound ), d=True)
				
				#If the attribute is connected, we go further in dept with the searching
				if connections != None:	
					#We are assuming that the mesh is connected with another hmCompound as input...
					
					#Remove from the layer adding
					self.originalMeshForLayer.remove(inputMesh)

					print 'connections are', connections
					#Here we get the connected compound
					connectedCompound = connections[0]		
					
					#Connect the found compound to the current one
					# try:
					# 	cmds.connectAttr('%s.%s' % (connectedCompound, gVar.outCompoundToMesh), '%s.%s' % (self.compoundNode, gVar.outCompoundToMesh))
					# except:
					# 	cmds.warning('Something went wrong during the connection process...')
					
					print 'connectedCompound', connectedCompound, 'will be connected to', self.compoundNode
					# We connect the relative attribute to the other compound
					cmds.connectAttr('%s.%s' % (connectedCompound, gVar.previewAndNextCompound), '%s.%s' % (self.compoundNode, gVar.previewAndNextCompound))
						
					#Display a warning message
					cmds.warning('%s mesh is the result of another Hard Mesh Operation, forcing smooth to 0.' % inputMesh )
					
					#Print messages
					# print 'connected:'
					# print '%s.%s' % (connectedCompound, gVar.outCompoundToMesh)
					# print '%s.%s' % (self.compoundNode, gVar.outCompoundToMesh)
					
					#The value of the smooth
					return True
					
				#If our attribute is not connected, just return it as false, since means that is no more connected...
				else:
					# print 'nothing connected'
					return False
		else:
			# print 'not from network'
			return False
				
				
	
		# print 'ignore check if came from network'
				
	def getNumber(self, currentNetworks ):
		numList = []
		for network in currentNetworks:
			if hmUtil.hasNumbers( network ):
				numList.append( network[len(network)-1] )

			
		if len(numList) != 0:
			return int( list( reversed(sorted(numList)) )[0] ) +1
		else:
			return 1
	
	def createShader(self, name):
		'''
		Create a shader of the specified type, if another equal shader doesn't exist
		'''
		#Creiamo un filtro per vedere se nel mondo ci sono gia layers
		shaderFilter = cmds.itemFilter( byName='%s' % name )
		existingShader = cmds.lsThroughFilter( shaderFilter )
		
		if existingShader == None:
			#f nothing with the same name exists...
			if name == gVar.darkPhong:
				cmds.shadingNode( 'phongE', asShader=True, n = name )
				#Set the appareance
				cmds.setAttr( '%s.color' % name, 0.15, 0.15, 0.15, type='double3' )
				cmds.setAttr( '%s.whiteness' % name, 0.65, 0.65, 0.65, type='double3' )
				cmds.setAttr( '%s.specularColor' % name, 1, 1, 1, type='double3' )
				cmds.setAttr( '%s.highlightSize' % name, 0.6)
			if name == gVar.reflectionShader:
				cmds.warning('no shader defined for reflection type')
			if name == gVar.zebraShader:
				cmds.warning('no shader defined for zebra type')
	
	def controlShaderOptionVar(self):
		'''
		This method controls if the shader option var exist, if yes
		the name of the shader is used as self.hmShader.
		'''
		exists = cmds.optionVar(exists = gVar.shaderOptionVarName)
		if exists == 1:
			self.hmShader = cmds.optionVar( q=gVar.shaderOptionVarName )
			# print 'option var found'
		else:
			# print 'no option var, using default lambert'
			pass
			
	def createLayer(self, layerName, color, setType ):
		'''
		This method checks if the layer exist, and if not it creates it.
		'''
		
		#Creiamo un filtro per vedere se nel mondo ci sono gia layers
		layerFilter = cmds.itemFilter( byName='%s' % layerName )
		existingLayer = cmds.lsThroughFilter( layerFilter )
		
		if existingLayer is None:
			#Creiamo un display layer per le mesh originarie, e salviamo il suo nome in un attributo nascosto per ricavarlo semplicemente
			cmds.createDisplayLayer( name = layerName )
			cmds.setAttr( '%s.color' % layerName, color)
			cmds.setAttr( '%s.displayType' % layerName, setType)
	

	def getColorFromLayer(self, layer, inGeo):
		'''
		We get the color of the specified layer and we assign it to the shape of the selected object.
		This is required since we have activeted the override in the shape, losing in this way the color override from layer assigned to transform level.
		'''
		shape = cmds.listRelatives( inGeo, shapes = True ) [0]
		color = cmds.getAttr( '%s.color' % layer )
		cmds.setAttr( '%s.overrideColor' % shape, color)


		
	def connecColorToShape( self, layer, inGeo ):
		'''
		We connect directly the layer color to shape
		'''
		# =======================================================================================
		# ==================== THIS WAS GIVING PROBLEMS WITH UNDO, DONT USE IT ==================== 
		# =======================================================================================

		shape = cmds.listRelatives( inGeo, shapes = True ) [0]
		cmds.connectAttr( '%s.color' % layer, '%s.overrideColor' % shape, f=True)
		# try:
			# shape = cmds.listRelatives( inGeo, shapes = True ) [0]
			# cmds.connectAttr( '%s.color' % layer, '%s.overrideColor' % shape)
		# except:
			# pass
		
	def addHiddenAttr( self, addAttrTo, attrName, attrValue):
		#Var
		attrNum = 0
		
		#Verifichiamo se questo Attributo non esista gia, se esiste, lo creeremo con un numero incrementale
		customAttrs = cmds.listAttr( addAttrTo, userDefined=1 )
		if customAttrs is not None:
			currentAttrCounter = 0
			for attr in customAttrs:
				if attrName in attr:
					currentAttrCounter =+1
					attrNum = currentAttrCounter
		
		attrName = 	'%s_%s' % (attrName, attrNum)		
		
		cmds.addAttr( addAttrTo, longName= attrName, at= 'long', defaultValue=attrValue, k=False )		
		
		return attrName
		
	
	def addAndConnectAttr( self, addAttrTo, attrName, connectAttrTo, attrValue, attrType,  minVal = None , maxVal = None, addPolicy = 'incrementIfExist', keyable = True):
		'''
		Add and connect attributes of type: bool, long, double.
		If an attribute already exist, it will be added the attribute +incrementel number
		Other attribute type are not supported
		
		supported addPolicy string:
		
						-incrementIfExist:	the attribute will be created with an incremental number, and the number will be increased if an attribute with the same name will be found
						-skipIfExist:		the name of the attribute will be the same provided, and if a conflict will be found, the creation will be skipped.
		'''
		#======= Define variables
		addPolicyString = addPolicy
		attrNum = 0
		attrExist = False
		keyableVal = keyable
		
		
		#======= Query if attribute exist
		customAttrs = cmds.listAttr( addAttrTo, userDefined=1 )
		
		if customAttrs is not None:
			currentAttrCounter = 0
			for attr in customAttrs:
				if attrName in attr:
					#print attrName, 'in', attr
					#Found already an existing attribute with the same name
					attrExist = True
					#Add a number if we need to add it anyway
					currentAttrCounter +=1
					attrNum = currentAttrCounter
		
		#======= Define the final name
		if addPolicyString == 'incrementIfExist':
			attrName = '%s_%s' % (attrName, attrNum)
		if addPolicyString == 'skipIfExist':
			attrName = attrName
		#print 'aggiungo l\' attributo', attrName
		
		#======= Add the attribute as specifyed in the options
		
		#We add the attribute if these conditions ade satisfied
		if addPolicyString == 'incrementIfExist' or  attrExist == False and addPolicyString == 'skipIfExist': 
			if minVal is None and maxVal is None:
				#print 'attributo senza limiti'
				cmds.addAttr( addAttrTo, longName= attrName , at= attrType, defaultValue=attrValue, k=keyableVal )
			if minVal is not None and maxVal is None:
				#print 'attributo con minimo'
				cmds.addAttr( addAttrTo, longName= attrName , at= attrType, min= minVal, defaultValue=attrValue, k=keyableVal )
			if maxVal is not None and minVal is None:
				#print 'attributo con massimo'
				cmds.addAttr( addAttrTo, longName= attrName, at= attrType, max= maxVal, defaultValue=attrValue, k=keyableVal )
			if maxVal and minVal is not None:
				#print 'attributo con minimo e massimo'
				cmds.addAttr( addAttrTo, longName= attrName , at= attrType, min= minVal, max= maxVal, defaultValue=attrValue, k=keyableVal )

			
		##======= Connect it to the provided input, if not empty
		#print  ' connectAttrTo != '', ' (connectAttrTo != '')
		if connectAttrTo != '':
			# print '%s.%s' % (addAttrTo, attrName), '%s' % connectAttrTo
			cmds.connectAttr('%s.%s' % (addAttrTo, attrName), '%s' % connectAttrTo)
		
		
		return attrName
		
		
	def connectTransform( self, inputA, inputB ):
		'''
		Creiamo un constraint per guidare le mesh smussate
		'''
		
		#Constraint
		parentNode = cmds.parentConstraint( inputA, inputB ) [0]
		scaleNode = cmds.scaleConstraint( inputA, inputB ) [0]

		
		#return matrixNode
		return parentNode, scaleNode
			
		
	def deleteIntermediateObjs(self, inputTransform):
		'''
		If an object has more than one shape, we delete the intermediate ones...
		'''

		#Troviamo le shape
		shapes = cmds.listRelatives(inputTransform, shapes = True, f=True)
		#Se abbiamo shapes...
		if len(shapes ) > 1:
			#Per ogni shape
			for shape in shapes:
				#Se la shape corrente e una mesh...
				if cmds.objectType( shape, isType = 'mesh'): 
					#Se la mesh e un intermediateObj
					isIntermediate = cmds.getAttr( '%s.intermediateObject' % shape )
					if isIntermediate:
						# print 'cancello la shape: %s' % shape
						cmds.delete( shape )
				
	def createPanelChoice(self, inputA0, inputA1, inputB0, inputB1 ):
		'''
		This method creates 2 choice nodes to use then the panel operation is called
		
		'''
		choiceNode1 = cmds.createNode ( 'choice', name = 'offsetted_to_not_offsetted' )
		choiceNode2 = cmds.createNode ( 'choice', name = 'not_offsetted_to_offsetted')
		
		#Connect the first input, to the first input of the choice, and so on..
		cmds.connectAttr( '%s.%s' % (inputA0[0], inputA0[1]), '%s.input[0]' % choiceNode1 )
		cmds.connectAttr( '%s.%s' % (inputA1[0], inputA1[1]), '%s.input[1]' % choiceNode1 )
		
		cmds.connectAttr( '%s.%s' % (inputB0[0], inputB0[1]), '%s.input[0]' % choiceNode2 )
		cmds.connectAttr( '%s.%s' % (inputB1[0], inputB1[1]), '%s.input[1]' % choiceNode2 )
		
		#Connect selectors
		cmds.connectAttr( '%s.selector' % choiceNode1, '%s.selector' % choiceNode2 )
		
		return [choiceNode1, choiceNode2]
		
		
		
	def createDirectionChoice( self, attributeA, attributeB, attributeC, attributeD, attributeE, attributeF, attributeToControl, compoundNodeForAttributes ):
		'''
		This method create a choice node for inverting the offset value based on the input attributes that have positive or negative value
		'''
		choiceNode = cmds.createNode ( 'choice' )
		#Colleghiamo l' operazione corrente con il selector
		cmds.connectAttr( '%s.%s' % ( compoundNodeForAttributes, self.operationType ), '%s.selector' % choiceNode )
		for id,attr in enumerate( [ attributeA, attributeB, attributeC, attributeD, attributeE, attributeF ] ):
			cmds.connectAttr( '%s.%s' % ( compoundNodeForAttributes, attr ), '%s.input[%s]' % (choiceNode, id) )
		#Colleghiamo il risultato del nostro svarione
		cmds.connectAttr( '%s.output' % choiceNode, attributeToControl )
		
		return choiceNode
	
	
	def createSmoothNode(self, inputMesh, smoothVal, mode = 0):
		'''
		This method create a polySmooth node for allowing the meshes to be smoothed before any hm related computation.
		This will allow the use to leave the original input mesh with the default subdivision layer.
		'''
		returnNode = None 

		
		#Create a poly smooth..
		smoothNode = self.createProblematicNode( inputMesh, 'polySmooth' )
		#Set the smooth to open subdiv
		cmds.setAttr('%s.subdivisionType' % smoothNode, 2)
		#Create the attributes for smooth control 	
		self.addAndConnectAttr( inputMesh, self.smoothAttrName, '%s.divisions' % smoothNode, smoothVal, 'long', addPolicy = 'skipIfExist', minVal = 0 ) #We add it to the mesh	
		#Get the original shape 
		shape = cmds.listRelatives( inputMesh, fullPath=True, s=True) [0]
		#Connect it as an input of the polySmooth
		cmds.connectAttr( '%s.outMesh' % shape, '%s.inputPolymesh' % smoothNode)
		#If the mode is 0, add also the output mesh.. this makes maya not freeze 
		if mode == 0:
			#Now create the output mesh
			outputMeshShape = cmds.createNode('mesh', parent = inputMesh )
			#And connect it to the smooth out 
			cmds.connectAttr( '%s.output' % smoothNode, '%s.inMesh' % outputMeshShape)
			#Set the out mesh as an intermediate object
			cmds.setAttr('%s.intermediateObject' % outputMeshShape, 1)
			#Save the return as mesh
			returnNode = [outputMeshShape,  'outMesh']
		# The mode 1 is used by the intersection curves for visualizing the smoothed mesh
		if mode == 1:
			cmds.setAttr('%s.intermediateObject' % shape, 1)
			#Now create the output mesh
			outputMeshShape = cmds.createNode('mesh', parent = inputMesh )
			#And connect it to the smooth out 
			cmds.connectAttr( '%s.output' % smoothNode, '%s.inMesh' % outputMeshShape)
			#Set the out mesh as an intermediate object
			#Save the return as mesh
			returnNode = [outputMeshShape,  'outMesh']
		else:
			returnNode = [smoothNode, 'output']


		
		return returnNode

		
	def createMeshOffset(self, inMatrix, inMeshAndConnection, offsetValue, compoundNodeForAttributes ):
		'''
		This method creates the offset with a poly move vertex.
		'''

		#Creiamo due nodi di polyMooveVertex collegati alla mesh data.
		offsetNode = cmds.rename( self.createProblematicNode( inMatrix, 'polyMoveVertex' ), 'mainOffset' )
		offsetNodeOpposite = cmds.rename( self.createProblematicNode( inMatrix, 'polyMoveVertex' ), 'oppositeOffset' ) #This node will be always the opposite of the first one, used in the panel creation.
		
		#Inseriamo un multiplyDivide per invertire i valori all occorrenza
		multNode = cmds.createNode( 'multiplyDivide' )
		invertMultNode = cmds.createNode( 'multiplyDivide' ) #This node is used for always inverting the values of the first poly move vertex
		
		#Create the connections for invert the offset
		cmds.connectAttr( '%s.localTranslateZ' % offsetNode, '%s.input1X' % invertMultNode ) #The offset value of the first offset to input of the invert
		cmds.connectAttr( '%s.outputX' % invertMultNode, '%s.localTranslateZ' % offsetNodeOpposite ) #The output from invert to the ovvset value of the second offsetNode 
		cmds.setAttr( '%s.input2X' % invertMultNode, -1 ) #Set the value for inverting incoming values 
		
		#Create the related offset attribute on the compound node
		self.addAndConnectAttr( compoundNodeForAttributes, self.offsetAttrName , '%s.input1X' % multNode, offsetValue, 'double', minVal = 0 )
		cmds.connectAttr( '%s.outputX' % multNode, '%s.localTranslateZ' % offsetNode )
		
	
		#Create the connection with the smooth node
		cmds.connectAttr( '%s.%s' % (inMeshAndConnection[0], inMeshAndConnection[1]), '%s.inputPolymesh' % offsetNode)
		cmds.connectAttr('%s.%s' % (inMeshAndConnection[0], inMeshAndConnection[1]), '%s.inputPolymesh' % offsetNodeOpposite)
			
			
		return offsetNode, multNode, offsetNodeOpposite
	
	def createDirectionChoicesParameters(self, nodeToAddTo):
		'''
		We create some values that will be used for flipping the directions of offset based on the chosen operation
		'''
		#aggiungiamo dei parametri per controllare la direzione delle curve di offset
		self.unionVal = self.addHiddenAttr( nodeToAddTo, 'ouUnion', 1) #The plus means that the offset for the union will be for both on the normal direction
		#A-B
		self.diffAVal = self.addHiddenAttr( nodeToAddTo, 'ouDifferenceA', -1) #The minus means that the offset for the A side, in difference, will be opposite
		self.diffBVal = self.addHiddenAttr( nodeToAddTo, 'ouDifferenceB', 1) #Same concept...
		#B-A
		self.diffAVal1 = self.addHiddenAttr( nodeToAddTo, 'ouDifferenceA', 1) #Same concept...
		self.diffBVal1 = self.addHiddenAttr( nodeToAddTo, 'ouDifferenceB', -1) #Same concept...
		#Intersect
		self.intVal = self.addHiddenAttr( nodeToAddTo, 'ouIntersection', -1) #The minus means that the offset for the difference will be for both on the OPPOSITE normal direction
		#Panel A-B
		self.panelAVal = self.addHiddenAttr( nodeToAddTo, 'ouPanelA', 1)
		self.panelBVal = self.addHiddenAttr( nodeToAddTo, 'ouPanelB', -1)
		#Panel B-A
		self.panelAVal1 = self.addHiddenAttr( nodeToAddTo, 'ouPanelA', -1)
		self.panelBVal1 = self.addHiddenAttr( nodeToAddTo, 'ouPanelB', 1)
		
		
		
	def createCurveOffset(self, curve, radius ):
		'''
		creiamo due curve di offset per la curva data, uno positivo e l' altro negativo
		'''
		#Eventualmente valutiamo se serve cambiare la normale in base alla direzione di proiezione....
		posOffset = cmds.offsetCurve( curve, distance = radius, ugn = False )
		negOffset = cmds.offsetCurve( curve, distance = -radius, ugn = False )
		
		return [posOffset[0], negOffset[0], posOffset[1], negOffset[1] ]
	
	def createProblematicNode(self, inputMesh, node ):


		#Duplichiamo l oggetto e usiamo il nodo marcio
		tmpDup = cmds.duplicate( inputMesh, name = 'hmTmp' ) [0]

		if node is 'polySmooth':
			nodeUnnamed = cmds.polySmooth( tmpDup, dv=0, kb=False )
			problematicNode = cmds.rename( nodeUnnamed, gVar.polySmoothName )

		if node is 'polyMoveVertex':
			problematicNode = cmds.polyMoveVertex( tmpDup )
		#duplichiamo il nodo 
		problematicNode = cmds.duplicate( problematicNode ) [0]
		#Cancelliamo il workarount
		cmds.delete( tmpDup )
		#Creiamo la connessione che volevamo
		#cmds.connectAttr( '%s.worldMesh[0]' % shape, '%s.inputPolymesh' % problematicNode)
		return problematicNode		

		
	def overrideColor (self, listOfInputs, color=0,  setTo = 1):

		lenght = len(listOfInputs)

		if (setTo == 0):
			for x in range(0,lenght,1):
				cmds.setAttr(listOfInputs[x] + '.overrideEnabled', 0)

		if (setTo == 1):
			for x in range(0,lenght,1):
				cmds.setAttr(listOfInputs[x] + '.overrideEnabled', 1)
				cmds.setAttr(listOfInputs[x] + '.overrideColor', color)

	def createIntersectionNodes(self, inputMeshTransformA, inputA, inputMeshTransformB, inputB, name, showCrv = False):
			'''
			Questa simpaticissima funzione crea i nodi di intersezione necessari...
			'''
			
			
			#Crea un nodo paintFxToon per avere le intersezioni		
			paintFxToon = cmds.createNode( 'pfxToon' )
			transformToDel = cmds.listRelatives( paintFxToon, fullPath=True, p=True) [0]
			#Riordina la shape in modo da non avere l' outliner sporco
			cmds.parent(paintFxToon, self.intersectorsGrp, r=True, s=True )
			cmds.delete( transformToDel )
			#Impostiamo le opzioni del nodo
			cmds.setAttr( '%s.profileLines' % paintFxToon, 0)
			cmds.setAttr( '%s.borderLines' % paintFxToon, 0)
			cmds.setAttr( '%s.creaseLines' % paintFxToon, 0)
			cmds.setAttr( '%s.intersectionLines' % paintFxToon, 1)
			cmds.setAttr('%s.degree' % paintFxToon, 1)
				
			#Creiamo le connessioni
			cmds.connectAttr( '%s.%s' % (inputA[0], inputA[1]), '%s.inputSurface[0].surface' % paintFxToon)
			cmds.connectAttr( '%s.worldMatrix[0]' % inputMeshTransformA, '%s.inputSurface[0].inputWorldMatrix' % paintFxToon)
			
			cmds.connectAttr( '%s.%s' % (inputB[0], inputB[1]), '%s.inputSurface[1].surface' % paintFxToon)
			cmds.connectAttr( '%s.worldMatrix[0]' % inputMeshTransformB, '%s.inputSurface[1].inputWorldMatrix' % paintFxToon)

			
			#Collega le curve di output nell input del compound.
			intersectionNum = cmds.getAttr( '%s.outMainCurveCount' % paintFxToon )

			#Listiamo quante connessioni sono collegate all' in curve. se è vuoto darà 0, quindi perfetto per collegare al primo slot.
			#Mano a mano si otterrà il numero giusto di connessioni in base alla lunghezza, e se dal nodo corrente, ci sono più  curve aggiungeremo dentro al loop un numero incrementale.
			# conn = cmds.listConnections('%s.inCurves' % compoundNode)	
			inCurvesId  = 0 #len(conn) if type(conn) is list else 0
			
			
			createCurve = 1
			
			for index,intersection in enumerate( range(0,intersectionNum) ):
				#Connessioni dirette	
				#cmds.connectAttr( '%s.outMainCurves[%s]' % (paintFxToon, index), '%s.InCurves[%s]' % (compoundNode, inCurvesId))
				inCurvesId +=1
				
				if createCurve == 1:
					currCrv = cmds.createNode( 'nurbsCurve' )
					curvTransform = cmds.listRelatives( currCrv, p=True) [0]
					#Out Curve to curveShape
					cmds.connectAttr( '%s.outMainCurves[%s]' % (paintFxToon, index), '%s.create' % currCrv)
					#Hide it
					cmds.setAttr('%s.visibility' % curvTransform, showCrv)
					#Gerarchia
					cmds.parent( curvTransform, self.curvesGrp)
					#Nome
					cmds.rename( curvTransform, '%s_crv' % name ) 
					
					
			
			#Nome
			paintFxToon = cmds.rename( paintFxToon, name ) 
			
			return paintFxToon
	
	def createMeshToHmOffsetConnections(self, inMatrix, inMeshAndConnection, slot):
		'''
		THis create the connection mesh/smooothNode >> orderToSplit >> compound
		'''
		# Mesh to offset
		cmds.connectAttr('%s.%s' % (inMeshAndConnection[0], inMeshAndConnection[1]), '%s.%s[%s]' % (self.offsetNode, gVar.inMeshes, slot) )		
		cmds.connectAttr('%s.worldMatrix[0]' % inMatrix, '%s.%s[%s]' % (self.offsetNode, gVar.inMeshesMatrix, slot) )
		
		
	def createMeshToCompoundConnections(self, inMatrix, inMeshAndConnection, slot):
		'''
		THis create the connection mesh/smooothNode >> orderToSplit >> compound
		'''
		#Colleghiamo gli output degli smooth al compound, e le outMatrix all' inMatrixA
		cmds.connectAttr('%s.%s' % (inMeshAndConnection[0], inMeshAndConnection[1]), '%s.%s[%s]' % (self.orderToSplit, gVar.inMeshes, slot) )
			
		cmds.connectAttr('%s.worldMatrix[0]' % inMatrix, '%s.%s[%s]' % (self.orderToSplit, gVar.inMeshesMatrix, slot) )
		#Now the connection from the orderToSplit to the Compound
		cmds.connectAttr('%s.%s[%s]' % (self.orderToSplit, gVar.outMeshesMatrix, slot), '%s.inMeshesMatrix[%s]' % (self.compoundNode, slot) )
		cmds.connectAttr('%s.%s[%s]' % (self.orderToSplit, gVar.outMeshes, slot), '%s.inMeshes[%s]' % (self.compoundNode, slot) )
	
	def createDirectCurvesConnection(self):
		'''
		This method connect the arrays directly to each other and create a curveVar Group for visualizing curves
		'''
		
		#Connect out curves from intersectors
		cmds.connectAttr( '%s.outMainCurves' % self.intersectionNode, '%s.inInterCurves' % self.orderToSplit )
		cmds.connectAttr( '%s.outMainCurves' % self.offsetANode, '%s.inOffsetCurvesA' % self.orderToSplit )
		cmds.connectAttr( '%s.outMainCurves' % self.offsetBNode, '%s.inOffsetCurvesB' % self.orderToSplit )
		
		# Connect out curves from order to split
		cmds.connectAttr( '%s.outCurves' % self.orderToSplit, '%s.inCurves' % self.compoundNode )
		
		
	def createCurvesConnectionHmOffset(self, compoundExist = True):
		'''
		This method validate the curves and connect them to the provided compound node
		'''
		removeOriginalCurves = True 
		
		name = 'postProcessedCrv'
		
		#======= INTERSECTION CURVES TO OFFSET ==================

		curveToOperateOn = self.validateCurves( cmds.listRelatives( self.curvesGrp, c=True, f=True) )	
		
		curveId = 0
		
		for index,curve in enumerate(curveToOperateOn):
			#Intersector to hmOffset
			cmds.connectAttr( '%s.local' % curve, '%s.%s[%s]' % (self.offsetNode, gVar.inCurves, curveId))
			# Important, increase the counter
			curveId +=1	

			
		#======= RESULT CURVES VISUAL OUTPUT ==================
		# getting the lenght of the attribute was not working.. assume that for every valid intersection 3 output curves will be created...
		numberOfOutCurve = len(curveToOperateOn) *3
		
		for x in range (numberOfOutCurve):
			#===== Post processed curves..
			currCrv = cmds.createNode( 'nurbsCurve' )
			curvTransform = cmds.listRelatives( currCrv, p=True) [0]
			# Connect the curve in the create
			cmds.connectAttr( '%s.%s[%s]' % (self.offsetNode, gVar.outCurves, x), '%s.create' % currCrv, force=True )

			#======= DIRECT CURVES CONNECTION, ARRAY SKIP ==================
			if compoundExist:
				# if required, direct connect each curve
				if self.noArray:
					cmds.connectAttr( '%s.%s[%s]' % (self.offsetNode, gVar.outCurves, x), '%s.%s[%s]' % (self.compoundNode, gVar.inCurves, x))



			#Gerarchia
			cmds.parent( curvTransform, self.curvesGrp)
			#Nome
			cmds.rename( curvTransform, '%s_crv' % name ) 			
		

		#======= ARRAY CURVES CONNECTION, MAKES COMPUTATIONS WEIRD ==================
		# Chose if we want to connect the curves as array or as single elements
		if compoundExist:
			if not self.noArray:
				cmds.connectAttr( (self.offsetNode + '.' + gVar.outCurves ), (self.compoundNode + '.' + gVar.inCurves ) )
			
			
		
	def createCurvesConnection(self):
		'''
		This method validate the curves and connect them to the provided compound node
		'''
		removeOriginalCurves = True 
		
		name = 'postProcessedCrv'
		
		#=======GET CURVES
		#Usiamo direttamente le curve originali, in quanto internamente dovrebbero gia essere creati dei duplicati
		curveToOperateOn = self.validateCurves( cmds.listRelatives( self.curvesGrp, c=True, f=True) )	

		
		inCurvesId = 0
		for index,curve in enumerate(curveToOperateOn):
		
			#Curve Shape to order to split
			cmds.connectAttr( '%s.local' % curve, '%s.%s[%s]' % (self.orderToSplit, gVar.inCurves, inCurvesId))
			
			#Order to split to compound
			cmds.connectAttr( '%s.%s[%s]' % (self.orderToSplit, gVar.outCurves, inCurvesId), '%s.inCurves[%s]' % (self.compoundNode, inCurvesId))
			
			# == Now we have connected the input curves, we have to query how many output curves there are since the curve sequencer
			# may have changet the number.
			
			
			#===== Post processed curves..
			currCrv = cmds.createNode( 'nurbsCurve' )
			curvTransform = cmds.listRelatives( currCrv, p=True) [0]
			# cmds.connectAttr( '%s.local' % curve, '%s.inCurves[%s]' % (compoundNode, inCurvesId))
			cmds.connectAttr( '%s.%s[%s]' % (self.orderToSplit, gVar.outCurves, inCurvesId), '%s.create' % currCrv )
			#Gerarchia
			cmds.parent( curvTransform, self.curvesGrp)
			#Nome
			cmds.rename( curvTransform, '%s_crv' % name ) 			
		
			
			# Important, increase the counter
			inCurvesId +=1
			
		
		

	def validateCurves(self, curves):
		'''
		Get only curves with more than 0 cvs
		'''
		returnArray = []
		for curve in curves:
			spans = cmds.getAttr( '%s.spans' % curve )
			if spans > 0:
				returnArray.append(curve)
				
		return returnArray
		

class HmSplitConnections():
	def __init__(self):
		pass
# ======================================================================== 	
# ============================ DA PULIRE =================================	
class HmBlendConnections():
	'''
	This class create the connections for the blend Node
	'''
	def __init__(self):
		pass

	def performMeshMeshBlend(self, objectA, objectB, idListA, idListB ):
		'''
		This method will pass the right data from the selection to the right node connector
		'''
		# Save the selection
		sel = cmds.ls(sl=True)
		# Get 2 different lists
		returned = divideComponentListByObjects( sel )

		# Create the fill node	
		fillNode = cmds.createNode('hmFillNode')

		# And the output mesh
		resultMesh = cmds.createNode('mesh')
		
		# Get the object transform shape and the shape
		objA =  objectA
		objB = objectB
		shapeA = cmds.listRelatives(objA, s=True)[0]
		shapeB = cmds.listRelatives(objB, s=True)[0]


		# Connect the meshes to the node
		cmds.connectAttr( shapeA + '.outMesh', fillNode + '.inMeshes[0]' )
		cmds.connectAttr( shapeB + '.outMesh', fillNode + '.inMeshes[1]' )
		cmds.connectAttr( shapeA + '.worldMatrix[0]', fillNode + '.inMeshesMatrix[0]' )
		cmds.connectAttr( shapeB + '.worldMatrix[0]', fillNode + '.inMeshesMatrix[1]' )

		# print  'setto gli id', idListA
		# And set the components list
		cmds.setAttr( fillNode + '.edgeLoopA',  *([len(idListA)] + idListA), type = 'componentList')
		cmds.setAttr( fillNode + '.edgeLoopB',  *([len(idListB)] + idListB), type = 'componentList')



		# Connect the output of the node to the new mesh
		cmds.connectAttr( fillNode + '.outputMesh', resultMesh + '.inMesh' )

		#================ SHADER ASSIGNMENT
		cmds.select( resultMesh, r=True )
		cmds.hyperShade( assign = 'lambert1 ')
		cmds.select(cl=True)



		# Save the selection
		# sel = cmds.ls(sl=True)
		# # Get 2 different lists
		# returned = divideComponentListByObjects( sel )

		# # Create the fill node	
		# fillNode = cmds.createNode('hmFillNode')

		# # And the output mesh
		# resultMesh = cmds.createNode('mesh')
		
		# # Get the object transform shape and the shape
		# objA =  returned[0][0].split('.')[0]
		# shapeA = cmds.listRelatives(objA, s=True)[0]
		# objB = returned[1][0].split('.')[0]
		# shapeB = cmds.listRelatives(objB, s=True)[0]

		# # Now format the list of components in the way that maya wants
		# outListA = [x.split('.')[1] for x in returned[0] ]
		# outListB = [x.split('.')[1] for x in returned[1] ]

		# # Connect the meshes to the node
		# cmds.connectAttr( shapeA + '.outMesh', fillNode + '.inMeshes[0]' )
		# cmds.connectAttr( shapeB + '.outMesh', fillNode + '.inMeshes[1]' )
		# cmds.connectAttr( shapeA + '.worldMatrix[0]', fillNode + '.inMeshesMatrix[0]' )
		# cmds.connectAttr( shapeB + '.worldMatrix[0]', fillNode + '.inMeshesMatrix[1]' )

		# # And set the components list
		# cmds.setAttr( fillNode + '.edgeLoopA',  *([len(outListA)] + outListA), type = 'componentList')
		# cmds.setAttr( fillNode + '.edgeLoopB',  *([len(outListB)] + outListB), type = 'componentList')
		# #print outListA
		# #print outListB

		# # Connect the output of the node to the new mesh
		# cmds.connectAttr( fillNode + '.outputMesh', resultMesh + '.inMesh' )		

# NE HAI SCRITTA UNA MOLTO MEGLIO DENTRO AL HM BLEND OPTIONS		
def divideComponentListByObjects(componentSelection):
	'''
	NE HAI SCRITTA UNA MOLTO MEGLIO DENTRO AL HM BLEND OPTIONS
	questa funzione divide una lista di componenti di più mesh in N lste ognuna
	contenente soltanto i componenti di una singola mesh.
	'''
	
	## Nuove Variabili
	aComponents = []
	bComponents = []
	returnLists = []
	#dati passati dal perform
	cmds.select( componentSelection, r=True)
	objects = cmds.ls( hl=True)	
	edges = cmds.ls(componentSelection, fl=True)
	
	# print objects

	## Error check
	if len(objects) != 2:
		# print objects
		cmds.error( 'hey you, this commands wants 2 border edges, of 2 different meshes!! ')

	## Conversione in stringhe
	objA_STR = objects[0].encode('utf-8')
	objB_STR = objects[1].encode('utf-8')


	## loop per riempire le due liste
	for edge in edges:    
		aSerch = re.match( '\\b%s\\b.+' % objA_STR, edge.encode('utf-8') )
		bSerch = re.match( '\\b%s\\b.+' % objB_STR, edge.encode('utf-8') )
		if aSerch != None:
			aComponents.append(aSerch.group(0))
		if bSerch != None:
			bComponents.append(bSerch.group(0))
	
	#print aComponents
	#print bComponents
	
	returnLists = [aComponents] + [bComponents]
	return returnLists