#!/usr/bin/env python
# -*- coding: utf-8 -*-

import maya.cmds as cmds
# General modules
import hmScripts.UI.hmGlobalVariables as gVar
# Ui Modules
import hmScripts.UI.hmUiCommonUtils as uiUtil
# Creation modules
import hmScripts.Main.hmSelectionHandling as hmSelect
import hmScripts.Main.hmNodeConnector as hmNodesConnection

reload(gVar)
reload(uiUtil)
reload(hmSelect)
reload(hmNodesConnection)



class IntersectionTubesOptionsUI():
	def __init__(self):
		'''
		In the init function we query the option var and get the values.
		Otherwise we create default values for the interface
		'''
		#Tooltips
		self.tooltipArray = {
		'MeshSetupHmOffset' : 'Use the hmOffsetCurveNode',
		'MeshSetumMayaOffset' : 'Use maya nodes for offsetCurves',
		'MeshSetup1' : 'Setup 2 meshes for blending!\nKeep in mind that you can have multiple shell for each mesh.',
		'MeshSetup2' : 'Create a full setup, good for have a better idea of the result, \nthanks to the boolean preview, and keep de input mesh low res.\nYou can solve hmSetup one after each other in the solving process',
		'MeshSetup3' : 'Create only the cutting curves, without boolean preview and acting on the original mesh density.\nIt\'s lighter to edit but gives to you no preview',
		'MeshSetup4' : 'The offset of the curve from the A side.',
		'MeshSetup5' : 'The offset of the curve from the B side.',
		'MeshSetup6' : 'The smooth used on the first selected object.',	
		'MeshSetup7' : 'The smooth used on the first selected object.',
		}





		#We define here the default values for the options
		self.hmOpt_defaultOffsetType = 2 #Means maya offset
		self.hmOpt_defaultOperType = 1
		self.hmOpt_defaultOffsetVal = 0.1


		#We store the values in these list with 2 entry: the first one is the string for the name, and the second one is the value
		self.hmOpt_offsetType = ['hmOpt_offsetType', self.hmOpt_defaultOffsetType]
		self.hmOpt_operationType = ['hmOpt_operationType', self.hmOpt_defaultOperType]
		self.hmOpt_offsetVal = ['hmOpt_offsetVal', self.hmOpt_defaultOffsetVal]

		self.optionsList = [self.hmOpt_offsetType, self.hmOpt_operationType]
		# #For every option we check if an option var is defined, if yes we sobstitute the value with the default ones
		uiUtil.optionVarInit( self.optionsList )





	def createUiElements(self):
		'''
		This function implements the shared UI elements for mesh setup.
		We will use the same options both on the hmDockUi and from the floating panels options, for this reason we should avoid name conflicts of the widgets...
		'''

		#===== Mesh Setup
		#VERSION UPGRADE
		cmds.separator( style='in', height = 1 )
		# ====== DEVELOPING VAR 
		if gVar.develop == 1:
			cmds.rowLayout(nc = 3)
			self.offTypeOperation = cmds.radioButtonGrp('rOffType', labelArray2=['hmOffset', 'mayaOffset'], numberOfRadioButtons=2, columnWidth = ([1, 80], [2,100]), select = self.hmOpt_offsetType[1], an1= self.tooltipArray['MeshSetup2'], an2 = self.tooltipArray['MeshSetup3'] )	
		else:
			cmds.rowLayout(nc = 2)
		self.omOperation = cmds.optionMenu()
		cmds.menuItem( label = gVar.operationA )
		cmds.menuItem( label = gVar.operationB )
		cmds.menuItem( label = gVar.operationC )
		cmds.menuItem( label = gVar.operationD )
		cmds.menuItem( label = gVar.operationE )
		cmds.menuItem( label = gVar.operationF )
		#cmds.menuItem( label = 'Cut with B' )
		#Set the value
		cmds.optionMenu( self.omOperation, edit = True, select = self.hmOpt_operationType[1] )
		
		cmds.setParent( '..' )

		cmds.separator( style='in', height = 1 )
		self.fsOffset = cmds.floatSliderGrp( label ='Offset', field=True, precision = 3, columnWidth= ([2, 50], [3,100]), value = self.hmOpt_offsetVal[1] , max = 10, min = 0, ann = self.tooltipArray['MeshSetup4'] )	
		cmds.separator( height=10, style='in' )
		
		#Reset the parent for the next controls
		cmds.setParent( '..' )
		cmds.setParent( '..' )


	#================================ COMMANDS ========================================================
	def performCommand(self, *Args ):
		'''
		Perform the command
		'''		
		#Get the interface values
		self.getValuesFromInterface()
		

		#Init the sel handling
		selection = hmSelect.HmSelectionHandling()
		#Set custom messages
		selection.setErrorMessageIfZero('Please select 2 meshes to operate on.')
		selection.setErrorIfSmallThanMin('Please select 2 meshes to operate on.')
		selection.setErrorMessageIfOutLimit('Please select only 2 objects. You can have multiple shells for each mesh.')
		#Validate the selection and put the result on a var 
		sel = selection.validate(2,2,'mesh')
		
		
		#==== Perform setup
		hmIntersections = hmNodesConnection.HmOperationConnections()
		hmIntersections.tubeIntersection(sel[0], sel[1],0.1 )
	
	def saveOptionsValues(self):
		'''
		This method stores the current options in the optionVar 
		'''
		#Get the interface values
		self.getValuesFromInterface()
		#Save the options var
		uiUtil.optionVarSave( self.optionsList )


	def resetOptionsValues(self):
		'''
		This method reset the options with the defualt values
		'''
		#Reset the ui values
		cmds.optionMenu( self.omOperation, e=True, select= self.hmOpt_defaultOperType )
		cmds.floatSliderGrp( self.fsOffset, e=True, v= self.hmOpt_defaultOffsetVal )



	#================================ INTERFACE UPDATES ========================================================


	
	#================================ UTILITY ========================================================
	def getValuesFromInterface(self, *Args):
		'''
		This method store in the main variables the values of the interface
		'''
		# ====== DEVELOPING VAR 
		if gVar.develop == 1:
			self.hmOpt_offsetType[1] = cmds.radioButtonGrp( self.offTypeOperation, q=True, select=True ) 
			
		self.hmOpt_operationType[1] = cmds.optionMenu( self.omOperation, q=True, select=True )
		self.hmOpt_offsetVal[1]  = cmds.floatSliderGrp( self.fsOffset, q=True, v=True)
		

	def getMainTooltip(self):
		'''
		This method return the main tooltip hint for adding it to the main button
		'''
		return self.tooltipArray['MeshSetup1']




